package	jp.co.sds_corp.util.common;

import java.util.Vector;

/**
* データの受け渡しを行うキューの機能を提供します。
* 複数Thread間での非同期の変更を保証します。
*/
public class BlockQueue<T>
{
	Vector<T>	vec = new Vector<T>();

	/**
	* キューの内容が空かどうかを判定します。
	* @return	true:キューが空 false:キューにオブジェクトが存在する
	*/
	public synchronized boolean isEmpty()
		throws InterruptedException
	{
		return isEmpty( -1 );
	}

	/**
	* キューの内容が空かどうかを判定します。
	* 指定された待ち時間以内にキューにデータが来なければ
	* 一度returnします。
	* @param	wait	待ち時間(ミリ秒)。-1で無限
	* @return	true:キューが空 false:キューにオブジェクトが存在する
	*/
	public synchronized boolean isEmpty( long wait )
		throws InterruptedException
	{
		if ( vec.size() != 0 ) {
			return false;
		}
		if ( wait < 0 ) {
			wait();
		} else {
			wait( wait );
		}
		if ( vec.size() != 0 ) {
			return false;
		}
		return true;
	}

	/**
	* キューからデータを取得します。取得したデータは削除されません。
	* @return	取得したオブジェクト
	*/
	public synchronized T peek() {
		if ( vec.size() == 0 ) {
			return null;
		}
		T obj = vec.get( 0 );
		return obj;
	}

	/**
	* キューからデータを取得します。取得したデータは削除されます。
	* @return	取得したオブジェクト
	*/
	public synchronized T get() {
		if ( vec.size() == 0 ) {
			return null;
		}
		T obj = vec.get( 0 );
		vec.remove( obj );
		return obj;
	}

	/**
	* キュー内の指定された位置にあるデータを取得します。
	* @param	idx		返される要素の位置
	* @return	取得したオブジェクト
	*/
	public synchronized T get( int idx ) {
		return vec.get( idx );
	}

	/**
	* キューにデータを追加します。
	* @param	obj		追加するオブジェクト
	*/
	public synchronized void add( T obj ) {
		vec.add( obj );
		notifyAll();
	}

	/**
	* キューからデータを削除します。
	* @param	obj		削除するオブジェクト
	*/
	public synchronized void remove( T obj ) {
		vec.remove( obj );
	}

	/**
	* キューのデータをクリアします。
	*/
	public synchronized void clear() {
		vec.clear();
	}

	/**
	* キューのサイズを返します。
	* @return	キューのサイズ
	*/
	public synchronized int size() {
		return vec.size();
	}

}
