package	jp.co.sds_corp.util.common;

import java.text.AttributedCharacterIterator;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.TimeZone;

/**
* 日付のフォーマットと解析を、和暦を含めて行うための具象クラスです。
* フォーマット (日付 -&gt; テキスト)では、和暦の変換機能が追加されています。
* 解析 (テキスト -&gt; 日付)、および正規化については、{@link SimpleDateFormat}を参照してください。
* <P>※和暦の解析は、現在のところできません。</P>
* <strong>時刻フォーマット構文: </strong>
* <p> 時刻フォーマットを指定するには、時刻パターン文字列を使用します。
* このパターンでは、すべての ASCII 文字がパターン文字として使用されます。
* SimpleDateFormatで定義されている文字列に加えて以下の文字列が使用できます。
<blockquote>
<pre>
年号	意味				表示			例
------	-------				------------		-------
g	元号					(テキスト)		平成
Y	年(和暦)				(数値)			13
</pre>
</blockquote>
@see SimpleDateFormat
@see DateFormat
*/
public class WarekiFormat extends DateFormat
{
	private static final long serialVersionUID = 1L;
	/** フォーマット文字列 */
	private String		pattern;
	private Locale		loc;
	private TimeZone	tz;

	private Date defaultCenturyStart;
	private int defaultCenturyStartYear;

	int lineIndex;
	int lastIndex;

	/**
	* 指定されたロケールの指定されたパターンを使って初期化します。
	* @param fmt	フォーマット文字列
	*/
	public WarekiFormat( String fmt, Locale loc, TimeZone tz ) {
		pattern = fmt;
		this.loc = loc;
		this.tz = tz;

		calendar = Calendar.getInstance( tz, loc );
		numberFormat = NumberFormat.getInstance( loc );
		numberFormat.setGroupingUsed( false );

		if ( numberFormat instanceof DecimalFormat ) {
			((DecimalFormat )numberFormat).setDecimalSeparatorAlwaysShown( false );
		}

		numberFormat.setParseIntegerOnly( true );
		numberFormat.setMinimumFractionDigits( 0 );
		initializeDefaultCentury();
	}

	/**
	* 指定されたロケールの指定されたパターンを使って初期化します。
	* @param fmt	フォーマット文字列
	*/
	public WarekiFormat( String fmt, Locale loc ) {
		this( fmt, loc, TimeZone.getDefault() );
	}

	/**
	* デフォルトロケールの指定されたパターンを使って初期化します。
	* @param fmt	フォーマット文字列
	*/
	public WarekiFormat( String fmt ) {
		this( fmt, CmnLocale.getLocale() );
	}

	/**
	* デフォルトローケルのデフォルトパターンを使って初期化します。
	*/
	public WarekiFormat() {
		this( ( new SimpleDateFormat()).toPattern() );
	}

	/**
	* Date を日付/時刻文字列にフォーマットします。
	* @param date			日付/時刻文字列にフォーマットする Date
	* @param toAppendTo		返される時刻文字列のための文字列バッファ
	* @param pos			返された文字列内のフィールド位置を把握する。
	*						入力時は、必要であれば位置合わせフィールド。
	*						出力時は、位置合わせフィールドのオフセット。
	*/
	public StringBuffer format( Date date, StringBuffer toAppendTo,
								FieldPosition pos ) {
		// Initialize
		pos.setBeginIndex( 0 );
		pos.setEndIndex( 0 );

		// Convert input date to time field list
		calendar.setTime( date );

		boolean inQuote = false; // true when between single quotes
		char prevCh = 0; // previous pattern character
		int count = 0;  // number of time prevCh repeated
		for ( int i=0; i < pattern.length(); ++i ) {
			char ch = pattern.charAt( i );
			// Use subFormat() to format a repeated pattern character
			// when a different pattern or non-pattern character is seen
			if ( ch != prevCh && count > 0 ) {
				toAppendTo.append( subFormat( prevCh, count, toAppendTo.length(), pos ));
				count = 0;
			}
			if ( ch == '\'' ) {
				// Consecutive single quotes are a single quote literal,
				// either outside of quotes or between quotes
				if ((i+1) < pattern.length() && pattern.charAt(i+1) == '\'') {
					toAppendTo.append( '\'' );
					++i;
				} else {
					toAppendTo.append( '\'' );
					inQuote = !inQuote;
				}
			} else if (!inQuote
				&& (ch >= 'a' && ch <= 'z' || ch >= 'A' && ch <= 'Z')) {
				// ch is a date-time pattern character to be interpreted
				// by subFormat(); count the number of times it is repeated
				prevCh = ch;
				++count;
			} else {
				// Append quoted characters and unquoted non-pattern characters
				toAppendTo.append( ch );
			}
		}
		// Format the last item in the pattern, if any
		if ( count > 0 ) {
			toAppendTo.append( subFormat( prevCh, count, toAppendTo.length(), pos ));
		}
		SimpleDateFormat sdf = new SimpleDateFormat( toAppendTo.toString(), loc );
		if ( tz != null ) {
			sdf.setTimeZone( tz );
		}
		return sdf.format( date, new StringBuffer(), new FieldPosition( 0 ));
	}

	private final String patternChars = "gYe";
	// Map index into pattern character string to Calendar field number
	private static final int[] PATTERN_INDEX_TO_CALENDAR_FIELD = {
		Calendar.YEAR, Calendar.YEAR, Calendar.DAY_OF_WEEK
	};

	/**
	* フォーマット文字列を解析して、実際の変換を行うメソッドです。
	* @param ch				フォーマット文字列中の文字
	* @param count			文字の繰り返し数
	* @param beginOffset	フォーマット文字列中の位置
	* @param fieldPosition	返された文字列内のフィールド位置を把握する。
	*						入力時は、必要であれば位置合わせフィールド。
	*						出力時は、位置合わせフィールドのオフセット。
	*/
	private String subFormat( char ch, int count, int beginOffset,
								 FieldPosition pos )
		throws IllegalArgumentException
	{
		int     patternCharIndex = -1;
		int     maxIntCount = Integer.MAX_VALUE;
		String  current = "";

		if (( patternCharIndex=patternChars.indexOf( ch )) == -1 ) {
			char[] buf = new char[count];
			for ( int nLoop = 0; nLoop < count; nLoop++ ) {
				buf[nLoop] = ch;
			}
			return new String( buf );
		}

		int field = PATTERN_INDEX_TO_CALENDAR_FIELD[patternCharIndex];
		int value = calendar.get( field );
		switch ( patternCharIndex ) {
		case 0: // 'g' - 元号
			current = getGengo();
			break;
		case 1: // 'Y' - 和暦の年
			int wy = getWaYear();
			if ( count >= 4 ) {
				current = zeroPaddingNumber( wy, 4, maxIntCount );
			} else {
				current = zeroPaddingNumber( wy, 2, 2 );
			}
			break;

        case 2: // 'e' - 曜日
			if ( count == 2 ) {
				current = "E";
			} else if ( count == 3 ) {
				current = "E 曜";
			} else {
				current = "E 曜日";
			}
            break;

		default:
			break;
		}
		return current;
	}

	/** 各元号の開始年月日 */
	private GregorianCalendar[] sepcal = {
		new GregorianCalendar( 0,  0,  1 ),
		new GregorianCalendar( 1868,  0, 25 ),
		new GregorianCalendar( 1912,  6, 30 ),
		new GregorianCalendar( 1926, 11, 25 ),
		new GregorianCalendar( 1989,  0,  7 )
	};
	/** 各元号の名称 */
	private String[] Ggo = {
		"昔々", "明治", "大正", "昭和", "平成"
	};

    private static final String[] weekdays = {
        "", "日曜日", "月曜日", "火曜日", "水曜日", "木曜日", "金曜日", "土曜日"
    };

	private Calendar start;
	private int nGgoIdx;

	/**
	* 指定された日付の属する元号の開始年月日を取得する
	*/
	private void searchStart() {
		for ( int nLoop = 0; nLoop < sepcal.length; nLoop++ ) {
			if ( calendar.before( sepcal[nLoop] )) {
				nGgoIdx = nLoop - 1;
				return;
			}
		}
		nGgoIdx = Ggo.length-1;
	}
	/**
	* 指定された日付の元号名称を取得する
	*/
	private String getGengo() {
		searchStart();
		return Ggo[nGgoIdx];
	}
	/**
	* 指定された日付の元号年を取得する
	*/
	private int getWaYear() {
		searchStart();
		return calendar.get( Calendar.YEAR )
				- sepcal[nGgoIdx].get( Calendar.YEAR ) + 1;
	}

	/**
	* 元号文字列から、西暦に変換する
	*/
	private int encodeGengo( String gengo ) {
		for ( int i=0; i < Ggo.length; i++ ) {
			if ( gengo.equals( Ggo[i] )) {
				return sepcal[i].get( Calendar.YEAR );
			}
		}
		return sepcal[0].get( Calendar.YEAR );
	}

	/**
	* 元号年を西暦に変換する
	*/
	private int encodeWaYear( String yy, int pos, int c ) {
		String year;
		if ( c >= 4 ) {
			year = yy.substring( pos, ( pos + 4 ));
		} else {
			year = yy.substring( pos, ( pos + 2 ));
		}
		int y = Integer.parseInt( year ) - 1;
		return y;
	}

	/**
	* 元号文字列の長さを取得する
	*/
	private int getGgoLength() {
		return Ggo[0].length();
	}

	/**
	* 数値をフォーマットします。
	* @param value		フォーマットする数値
	* @param minDigits	最小桁数
	*/
	private String zeroPaddingNumber( long value, int minDigits, int maxDigits )
	{
		numberFormat.setMinimumIntegerDigits( minDigits );
		numberFormat.setMaximumIntegerDigits( maxDigits );
		return numberFormat.format( value );
	}

	private String makeSimpleDateFormatPattern( Object obj ) {
		boolean [] flags = { false, false, false };

		lineIndex = 0;
		lastIndex = pattern.length();

		String text = format( obj );
		ParsePosition pos = new ParsePosition( 0 );
		String patternString =  patternCheck( text, new StringBuffer(), pos, flags );

		return patternString;
	}

	/**
	* 日付/時刻文字列を解析します。
	* @param text	解析する日付/時刻文字列
	* @param pos	入力時は、解析を開始する位置。
	*				出力時は、解析が終了した位置、
	*				または解析が失敗した場合には開始位置
	*/
	public Date parse( String text, ParsePosition pos ) {

		this.calendar.clear();
		int defaultYear = this.calendar.get( Calendar.YEAR );

		boolean [] flags = { false, false, false };

		lineIndex = 0;
		lastIndex = pattern.length();

		String buffer = patternCheck( text, new StringBuffer(), pos, flags );
		if ( buffer == null ) {
			return null;
		}

		Date date;
		SimpleDateFormat sdt = new SimpleDateFormat( buffer );
		pos.setIndex( 0 );
		date = sdt.parse( text, pos );

		if ( date != null ) {
			Calendar calendar = (Calendar )this.calendar.clone();
			calendar.setTime( date );		// SimpleDateFormat-result

			if ( flags[0] || flags[1] ) {
				int year = calendar.get( 1 ) % 100;
				if ( !flags[0] ) {
					calendar.set( 1, year + defaultCenturyStartYear - 1 );
					this.calendar.setTime( calendar.getTime());
				} else {
					calendar.set( 1, this.calendar.get( 1 ) + year - 1 );
					this.calendar.setTime( calendar.getTime() );
				}
				return calendar.getTime();
			}
		}
		return date; 
	}

	public String patternCheck( String text, StringBuffer buffer,
			ParsePosition pos, boolean [] flags ) {

		String token;
		char ch;
		while (( ch = nextChar()) != '\0' ) {
			int index = pos.getIndex();
			switch ( charToIndex( ch )) {
				case 0:	// g
					token = make_g();
					buffer.append( text.substring( index, index + 2 ));
					pos.setIndex( index + 2 );
					this.calendar.set( Calendar.YEAR, encodeGengo( text.substring( index, index + 2 )));
					flags[0] = true;
					break;

				case 1:	// Y
					token = make_Y();
					try {
						int y = encodeWaYear( text, index, token.length());
						if ( y < 0 ) {
							if ( token.length() >= 4 ) {
								pos.setIndex( 0 );
								pos.setErrorIndex( index + 4 );
							} else {
								pos.setIndex( 0 );
								pos.setErrorIndex( index + 1 );
							}
							return null;
						}
						if ( token.length() >= 4 ) {
//							buffer.append( text.substring( index, index + 4 ));
							buffer.append( "00yy" );
						} else {
//							buffer.append( text.substring( index, index + 2 ));
							buffer.append( "yy" );
						}
					} catch ( NumberFormatException ne ) {
						if ( token.length() >= 4 ) {
							pos.setIndex( 0 );
							pos.setErrorIndex( index + 4 );
						} else {
							pos.setIndex( 0 );
							pos.setErrorIndex( index + 1 );
						}
						return null;
					}
					flags[1] = true;
					break;

				case 2:	// e
					token = make_e();
					buffer.append( token );
					pos.setIndex( index + token.length() );
					flags[2] = true;
					break;

				case 3:	// '
					token = make_sq();
					buffer.append( token );
					pos.setIndex( index + token.length() );
					break;

				default:
					buffer.append( ch );
					pos.setIndex( index+1 );
					break;
			}
			
		}
		return buffer.toString();
	}

	private char nextChar() {
		return ( lineIndex < lastIndex ) ? pattern.charAt( lineIndex++ ) : '\0';
	}

	private void initializeDefaultCentury() {
		calendar.setTime( new Date());
		searchStart();

		defaultCenturyStart = sepcal[nGgoIdx].getTime();
		defaultCenturyStartYear = sepcal[nGgoIdx].get( Calendar.YEAR );
	}

	static final String reserved = "gYe'";
	private int charToIndex( char ch ) {
		return reserved.indexOf( ch );
	}

	private String make_g() {
		char ch;
		StringBuffer buffer = new StringBuffer();

		buffer.append( 'g' );
		while (( ch = nextChar()) == 'g' ) {
			buffer.append( ch );
		}
		if ( ch != '\0' ) {
			lineIndex -= 1;
		}
		return buffer.toString();
	}

	private String make_Y() {
		char ch;
		StringBuffer buffer = new StringBuffer();

		buffer.append( 'Y' );
		while (( ch = nextChar()) == 'Y' ) {
			buffer.append( ch );
		}
		if ( ch != '\0' ) {
			lineIndex -= 1;
		}
		return buffer.toString();
	}

	private String make_e() {
		char ch;
		StringBuffer buffer = new StringBuffer();
		buffer.append( 'E' );

		int c = 1;
		while (( ch = nextChar()) == 'e' ) {
			c++;
		}

		if ( c == 2 ) {
		} else if ( c == 3 ) {
			buffer.append( " 曜" );
		} else {
			buffer.append( " 曜日" );
		}

		if ( ch != '\0' ) {
			lineIndex -= 1;
		}
		return buffer.toString();
	}

	private String make_sq() {
		char ch;
		StringBuffer buffer = new StringBuffer();

		buffer.append( '\'' );
		while (( ch = nextChar()) != '\'' ) {
			if ( ch == '\0' ) {
				break;
			}
			buffer.append( ch );
		}

		if ( ch == '\'' ) {
			buffer.append( ch );
		}
		return buffer.toString();
	}

	/**
	* フォーマットパターンを取得する
	*
	* @return フォーマットパターン
	*/
	public String toPattern()  {
		return pattern;
	}

	/**
	* 指定されたパターン文字列を、この日付フォーマットに適用します
	*
	* @param s この日付フォーマットのための新しい日付と時刻パターン
	*/
	public void applyPattern( String s ) {
		this.pattern = s;
	}

	/**
	* Objectをフォーマットし、AttributedCharacterliteratorを生成します。
	*
	* @param obj フォーマットするオブジェクト
	* @return フォーマットされた値を記述するAttributedCharacterIterator
	*/
	public AttributedCharacterIterator formatToCharacterIterator( Object obj ) {
		SimpleDateFormat sdf = new SimpleDateFormat(
			makeSimpleDateFormatPattern( obj ));
		return sdf.formatToCharacterIterator( obj );
	}

	public Object parseObject( String source, ParsePosition pos ) {
		return parse( source, pos );
	}

	public static void main( String [] args ) {
		try {
			ParsePosition pos = new ParsePosition( 0 );
			WarekiFormat fmt = new WarekiFormat( "gY/MM/dd eee" );
			Date date = (Date )fmt.parseObject( "平成01/10/10 金 曜", pos );
		} catch ( Exception ex ) {
		}
	}
}
