package	jp.co.sds_corp.util.net;

import java.net.InetAddress;

/**
* ICMPパケットの内容を保持するクラスです。
*/
public class ICMPPacket extends RawPacket
{
	/** タイプ */
	public enum Type
	{
		/** タイプ:echo reply */
		ECHO_REPLY( 0 ),
		/** タイプ:destination unreachable */
		DEST_UNREACHABLE( 3 ),
		/** タイプ:source quench */
		SOURCE_QUENCH( 4 ),
		/** タイプ:redirect */
		REDIRECT( 5 ),
		/** タイプ:echo request */
		ECHO_REQUEST( 8 ),
		/** タイプ:time exceeded */
		TIME_EXCEEDED( 11 ),
		/** タイプ:parameter problem */
		PARAMETER_PROBLEM( 12 ),
		/** タイプ:timestamp request */
		TIMESTAMP_REQUEST( 13 ),
		/** タイプ:timestamp reply */
		TIMESTAMP_REPLY( 14 ),
		/** タイプ:information request */
		INFO_REQUEST( 15 ),
		/** タイプ:information reply */
		INFO_REPLY( 16 ),
		/** タイプ:mask request */
		MASK_REQUEST( 17 ),
		/** タイプ:mask reply */
		MASK_REPLY( 18 );

		byte type;
		Type( int type ) {
			this.type = (byte )type;
		}
		byte getCode() {
			return this.type;
		}

		public static Type findType( byte code ) {
			Type ret = null;
			for ( Type type : Type.values()) {
				if ( type.getCode() == code ) {
					ret = type;
					break;
				}
			}
			return ret;
		}
	}

	byte	code;
	Type	type;
	byte[]	option = new byte[4];
	int		checksum;
	int		identifier;
	int		sequence;

	/**
	* ICMPパケットを生成します。
	* @param	data	バッファ
	* @param	length	バッファの長さ
	* @param	type	タイプ
	*/
	public ICMPPacket( byte data[], int length, Type type ) {
		super( data, length );
		setType( type );
	}

	/**
	* ICMPパケットを生成します。
	* @param	data	バッファ
	* @param	length	バッファの長さ
	* @param	type	タイプ
	* @param	code	コード
	*/
	public ICMPPacket( byte data[], int length, Type type, byte code ) {
		super( data, length );
		setType( type, code );
	}

	/**
	* ICMPパケットを生成します。
	* @param	data	バッファ
	* @param	length	バッファの長さ
	* @param	addr	送信先アドレス
	* @param	type	タイプ
	*/
	public ICMPPacket( byte data[], int length, InetAddress addr, Type type ) {
		super( data, length, addr );
		setType( type );
	}

	/**
	* ICMPパケットを生成します。
	* @param	data	バッファ
	* @param	length	バッファの長さ
	* @param	addr	送信先アドレス
	* @param	type	タイプ
	* @param	code	コード
	*/
	public ICMPPacket( byte data[], int length, InetAddress addr,
						Type type, byte code )
	{
		super( data, length, addr );
		setType( type, code );
	}

	/**
	* タイプとコードを設定します。
	* @param	type	タイプ
	* @param	code	コード
	*/
	public synchronized void setType( Type type, byte code ) {
		this.type = type;
		this.code = code; 
	}

	/**
	* タイプ(type)を設定します。
	* @param	var		設定するデータ
	*/
	public synchronized void setType( Type var ) {
		this.type = var ;
	}
	/**
	* タイプ(type)を取得します。
	* @return	typeの内容
	*/
	public synchronized Type getType() {
		return type;
	}

	/**
	* コード(code)を設定します。
	* @param	var		設定するデータ
	*/
	public synchronized void setCode( byte var ) {
		this.code = var ;
	}
	/**
	* コード(code)を取得します。
	* @return	codeの内容
	*/
	public synchronized byte getCode() {
		return code;
	}

	/**
	* チェックサム(checksum)を設定します。
	* @param	b1		設定するデータ
	* @param	b2		設定するデータ
	*/
	public synchronized void setChecksum( byte b1, byte b2 ) {
		this.checksum = calcInt( b1, b2 );
	}
	/**
	* チェックサム(checksum)を設定します。
	* @param	var		設定するデータ
	*/
	public synchronized void setChecksum( int var ) {
		this.checksum = var;
	}
	/**
	* チェックサム(checksum)を取得します。
	* @return	checksumの内容
	*/
	public synchronized int getChecksum() {
		return checksum;
	}

	/**
	* 識別コード(identifier)を設定します。
	* @param	b1		設定するデータ
	* @param	b2		設定するデータ
	*/
	public synchronized void setIdentifier( byte b1, byte b2 ) {
		this.identifier = calcInt( b1, b2 );
	}
	/**
	* 識別コード(identifier)を設定します。
	* @param	var		設定するデータ
	*/
	public synchronized void setIdentifier( int var ) {
		this.identifier = var;
	}
	/**
	* 識別コード(identifier)を取得します。
	* @return	identifierの内容
	*/
	public synchronized int getIdentifier() {
		return identifier;
	}

	/**
	* シーケンス番号(sequence)を設定します。
	* @param	b1		設定するデータ
	* @param	b2		設定するデータ
	*/
	public synchronized void setSequence( byte b1, byte b2 ) {
		this.sequence = calcInt( b1, b2 );
	}
	/**
	* シーケンス番号(sequence)を設定します。
	* @param	var		設定するデータ
	*/
	public synchronized void setSequence( int var ) {
		this.sequence = var;
	}
	/**
	* シーケンス番号(sequence)を取得します。
	* @return	sequenceの内容
	*/
	public synchronized int getSequence() {
		return sequence;
	}

	private int calcInt( byte b1, byte b2 ) {
		int nRet;
		if ( b2 < 0 ) {
			nRet = (b2 + 256) * 256;
		} else {
			nRet = b2 * 256;
		}
		nRet += b1;
		if ( b1 < 0 ) {
			nRet += 256;
		}
		return nRet;
	}
}
