package	jp.co.sds_corp.util.net;

import java.net.InetAddress;

/**
* IPパケットの基底クラスです。
*/
public class RawPacket
{
	IPHeader	header;
	byte[]		buf;
	int			offset;
	int			length;
	InetAddress	address;

	/**
	* 指定されたバッファでIPパケットを構築します。
	* @param	buf		バッファ
	* @param	length	バッファの長さ
	*/
	public RawPacket( byte buf[], int length ) {
		this( buf, 0, length );
	}    

	/**
	* 指定されたバッファでIPパケットを構築します。
	* @param	buf		バッファ
	* @param	offset	バッファのオフセット
	* @param	length	バッファの長さ
	*/
	public RawPacket( byte buf[], int offset, int length ) {
		setData( buf, offset, length );
		this.address = null;
		this.header = new IPHeader();
	}

	/**
	* 指定されたバッファでIPパケットを構築します。
	* @param	buf		バッファ
	* @param	offset	バッファのオフセット
	* @param	length	バッファの長さ
	* @param	address	送信先
	*/
	public RawPacket( byte buf[], int offset, int length,
						InetAddress address )
	{
		setData( buf, offset, length );
		setAddress( address );
		this.header = new IPHeader();
	}

	/**
	* 指定されたバッファでIPパケットを構築します。
	* @param	buf		バッファ
	* @param	length	バッファの長さ
	* @param	address	送信先
	*/
	public RawPacket( byte buf[], int length, InetAddress address ) {
		this( buf, 0, length, address );
	}    

	/**
	* このパケットのIPヘッダを返します。
	* IPヘッダは受信時にのみ設定されます。
	*/
	public synchronized IPHeader getHeader() {
		return header;
	}

	/**
	* このパケットの送信先を設定します。
	* @param	addr	送信先
	*/
	public synchronized void setAddress( InetAddress addr ) {
		address = addr;
	}
	/**
	* このパケットの送信先を返します。
	*/
	public synchronized InetAddress getAddress() {
		return address;
	}

	/**
	* このパケットのバッファを返します。
	*/
	public synchronized byte[] getData() {
		return buf;
	}

	/**
	* バッファのオフセットを返します。
	*/
	public synchronized int getOffset() {
		return offset;
	}

	/**
	* バッファの長さを設定します。
	* @param	length	バッファの長さ
	*/
	public synchronized void setLength( int length ) {
		if ( length > buf.length || length < 0 ) {
			throw new IllegalArgumentException( "illegal length" );
		}
		this.length = length;
	}
	/**
	* バッファの長さを返します。
	*/
	public synchronized int getLength() {
		return length;
	}

	/**
	* バッファを設定します。
	* @param	buf		バッファ
	* @param	offset	バッファのオフセット
	* @param	length	バッファの長さ
	*/
	public synchronized void setData( byte[] buf, int offset, int length ) {
		if ( length < 0 || offset < 0 || (( length + offset ) > buf.length )) {
			throw new IllegalArgumentException(
				"illegal length(" + length + ") or offset(" + offset + ")" );
		}
		this.buf = buf;
		this.length = length;
		this.offset = offset;
	}

	/**
	* バッファを設定します。
	* @param	buf		バッファ
	*/
	public synchronized void setData( byte[] buf ) {
		if ( buf == null ) {
			throw new NullPointerException( "buf is null" );
		}
		this.buf = buf;

		if ( length > buf.length ) {
			setLength( buf.length );
		}
	}

	/**
	* IPパケットのチェックサムを算出します。
	*/
	public synchronized int calcChecksum() {
		int sum = 0;

		int size = length;
		int i = 0;
		while ( size > 1 ) {
			int s = ((int )buf[i])+ ((int )buf[i+1]<<8);
			sum += s;
			i+=2;
			size -= 2;
		}

		if ( size!=0 ) {
			sum += buf[i]<<16;
		}

		sum = (sum & 0xffff) + (sum >> 16);
		sum = (sum & 0xffff) + (sum >> 16);

		return 0xffff ^ sum;
	}

}

