package	jp.co.sds_corp.util.script;

import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Iterator;
import java.util.Vector;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.Script;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
<a href="http://www.mozilla-japan.org/rhino/">Rhino</a>を使用して、JavaScriptを実行します。
<pre>
&lt;Script
	language="JavaScript"
	charset="euc-jp"
	file="table.js"
	debug="true"&gt;
	&lt;Include charset="euc-jp"
		file="common.js"/&gt;
&lt;/Script&gt;
</pre>
Include要素を指定すると、指定されたスクリプトをグローバルスコープに取り込みます。
<p>
スクリプトはXML内にも指定することも可能です。
<pre>
&lt;Script
	language="JavaScript"&gt;
	<EM>script...</EM>
	&lt;Include&gt;
		<EM>include script...</EM>
	&lt;/Include&gt;
&lt;/Script&gt;
</pre>
*/
public class JsParser extends AbstractScriptParser
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	Context cx;
	ScriptableObject global;
	Script	scr;
	Iterator<IConfiguration>	itrInc;
	Vector<Scriptable>	vecScope = new Vector<Scriptable>();
	Vector<Script>	vecScr = new Vector<Script>();

	protected JsParser() {
	}

	/**
	* 設定情報で初期化します。
	* @param    config  設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		itrInc = config.getChilds( "Include" );

		cx = Context.enter();
		global = cx.initStandardObjects();
		if ( isLoadOnInit()) {
			compile();
		}
	}

	/**
	* スクリプトに変数を設定します。
	* スクリプト内からは、名前でオブジェクトを参照します。
	* @param    name    変数名
	* @param    obj     オブジェクト
	*/
	public void setAttribute( String name, Object obj ) {
		if ( Context.getCurrentContext() == null ) {
			cx = Context.enter();
			global = cx.initStandardObjects();
		}
		Object jsObj;
		jsObj = Context.javaToJS( obj, global );
		ScriptableObject.putProperty( global, name, jsObj );
	}

	/**
	* スクリプトに設定されている変数を取得します。
	* @param    name    変数名
	* @return   オブジェクト
	*/
	public Object getAttribute( String name ) {
		if ( Context.getCurrentContext() == null ) {
			cx = Context.enter();
			global = cx.initStandardObjects();
		}
		Object jsObj;
		jsObj = ScriptableObject.getProperty( global, name );
		Object ret = Context.jsToJava( jsObj, String.class );
		return ret;
	}

	/**
	* スクリプトを実行します。
	* @return   実行結果
	*/
	public String execute() throws Exception {
		if ( Context.getCurrentContext() == null ) {
			cx = Context.enter();
			global = cx.initStandardObjects();
		}
		if ( !isCompiled()) {
			compile();
		}

		for ( int i = 0; i < vecScope.size(); i++ ) {
			ScriptableObject scope = (ScriptableObject )vecScope.get( i );
			Script scr = (Script )vecScr.get( i );
			scr.exec( cx, scope );
		}
		if ( scr == null ) {
			return null;
		}
		Object result = scr.exec( cx, global );
		if ( result == null ) {
			return null;
		}
		String str = Context.toString( result );
		return str;
	}

	/**
	* スクリプトをコンパイルします。
	*/
	public void compile() throws Exception {
		if ( getFile() != null ) {
			if ( isDebug()) {
				logger.debug( this, "compile [" + getFile() + "]" );
			}
			Reader rd = new InputStreamReader(
							new FileInputStream( getFile() ), getCharset() );
			scr = cx.compileReader( rd, getFile().getName(), 1, null );
		} else {
			if ( getScript() == null || getScript().length() == 0 ) {
				scr = null;
				vecScr.clear();
				return;
			}
			if ( isDebug()) {
				logger.debug( this, "compile [" + getScript() + "]" );
			}
			scr = cx.compileString( getScript(), "<cmd>", 1, null );
		}

		vecScope.clear();
		vecScr.clear();
		if ( itrInc == null ) {
			return;
		}
		Scriptable scope = null;
		Scriptable prev = null;
		while ( itrInc.hasNext()) {
			IConfiguration c = (IConfiguration )itrInc.next();
			String charset = c.getAttributes().get( "charset", getCharset() );
			String file = c.getAttributes().get( "file", null );
			String script = null;
			try {
				script = c.get();
			} catch ( Exception ex ) {
			}
			Script scr = null;
			if ( file != null ) {
				if ( getProperty() != null ) {
					file = getProperty().replaceValue( file );
				}
				Reader rd = new InputStreamReader(
								new FileInputStream( file ), getCharset() );
				if ( isDebug()) {
					logger.debug( this, "include [" + file + "]" );
				}
				scr = cx.compileReader( rd, file, 1, null );
			} else {
				if ( isDebug()) {
					logger.debug( this, "include [" + script + "]" );
				}
				scr = cx.compileString( script, "<cmd>", 1, null );
			}
			scope = cx.newObject( global );

			if ( prev != null ) {
				scope.setPrototype( prev );
				// scope.setParentScope( prev );
			}
			global.setPrototype( scope );
			// global.setParentScope( scope );
			prev = scope;
			vecScope.add( scope );
			vecScr.add( scr );
		}
		setCompiled( true );
	}

	/**
	* スクリプトの終了処理を行います。
	*/
	public void exit() {
		Context.exit();
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param    config  設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		while ( itrInc.hasNext()) {
			IConfiguration c = (IConfiguration )itrInc.next();
			config.addChild( c );
		}
	}

}
